/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package es.rickyepoderi.jmx.servlet;

import es.rickyepoderi.snmpjmxagent.ScalarAttributeDescription;
import es.rickyepoderi.snmpjmxagent.SnmpJmxModule;
import java.lang.management.ManagementFactory;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.annotation.WebListener;
import org.snmp4j.agent.MOGroup;
import org.snmp4j.agent.mo.MOAccessImpl;
import org.snmp4j.smi.Counter64;
import org.snmp4j.smi.OID;
import org.snmp4j.smi.OctetString;

/**
 * <p>Listener that loads the Hello JMX bean and maps the bean to the SNMP
 * server.</p>
 * 
 * @author ricky
 */
@WebListener
public class JmxListener implements ServletContextListener {

    /**
     * Attribute in the context to recover the Hello MBean.
     */
    public final static String JMX_BEAN = "es.rickyepoderi.snmpjmxagent.bean";
    
    /**
     * Attribute to recover the MOGroup assigned to the SNMP server.
     */
    public final static String SNMP_MO_GROUP = "es.rickyepoderi.snmpjmxagent.mog";
    
    /**
     * In the initialization of the context the Hello MBean is created and
     * assigned as an attribute (JMX_BEAN name). Besides the map for the 
     * SNMP is created and assigned to it using the singleton.
     * 
     * @param sce The servlet context.
     */
    @Override
    public void contextInitialized(ServletContextEvent sce) {
        try {
            System.err.println("Registering the bean...");
            ServletContext sc = sce.getServletContext();
            // register the Hello MBean in the JMX server, the bean is stored
            // in the context using the attribute JMX_BEAN
            Hello mbean = new Hello();
            String name = "es.sample.jmx:type=HelloMBean,name=" + sc.getContextPath();
            MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
            ObjectName objectName = new ObjectName(name);
            mbs.registerMBean(mbean, objectName);
            sc.setAttribute(JMX_BEAN, mbean);
            // register the Bean also in the SNMP server
            // for that the four attributes are described to get a proper mapping
            // the MOGroup is stored also in the context to unregister at destroy
            // This number is just to deploy the app two times and have another namespace
            int number = Math.abs(name.hashCode());
            ScalarAttributeDescription[] descs = new ScalarAttributeDescription[]{
                new ScalarAttributeDescription(new OID(new int[]{1, 3, 6, 1, 2, 1, number, 1, 0}), 
                    "Message", String.class, MOAccessImpl.ACCESS_READ_WRITE, new OctetString()),
                new ScalarAttributeDescription(new OID(new int[]{1, 3, 6, 1, 2, 1, number, 2, 0}), 
                    "Hellos", Long.class, MOAccessImpl.ACCESS_READ_ONLY, new Counter64()),
                new ScalarAttributeDescription(new OID(new int[]{1, 3, 6, 1, 2, 1, number, 3, 0}), 
                    "WorldHellos", Long.class, MOAccessImpl.ACCESS_READ_ONLY, new Counter64()),
                new ScalarAttributeDescription(new OID(new int[]{1, 3, 6, 1, 2, 1, number, 4, 0}), 
                    "NamedHellos", Long.class, MOAccessImpl.ACCESS_READ_ONLY, new Counter64())
            };
            SnmpJmxModule sjm = SnmpJmxModule.getInstance();
            MOGroup mog = sjm.registerBean(mbs, objectName, descs);
            sc.setAttribute(SNMP_MO_GROUP, mog);
        } catch (Exception e) {
            System.err.println("Error creating and registering the bean...");
            e.printStackTrace();
        }
    }

    /**
     * In the destruction of the context the bean is unregistered from the
     * JMX and SNMP server.
     * 
     * @param sce The servlet context.
     */
    @Override
    public void contextDestroyed(ServletContextEvent sce) {
        try {
            System.err.println("Unregistering the bean...");
            ServletContext sc = sce.getServletContext();
            // unregister the mbean from JMX server
            MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
            ObjectName name = new ObjectName("es.sample.jmx:type=HelloMBean,name=" + sc.getContextPath());
            mbs.unregisterMBean(name);
            // unregister from SNMP agent
            MOGroup mog = (MOGroup) sc.getAttribute(SNMP_MO_GROUP);
            SnmpJmxModule sjm = SnmpJmxModule.getInstance();
            sjm.unregisterBean(mog);
        } catch (Exception e) {
            System.err.println("Error unregistering the bean...");
            e.printStackTrace();
        }
    }
    
}
